#-*- coding: utf-8 -*-

import sys
import site

import mock
PACKAGE_NAME = u"Scroll pHAT HD"
PACKAGE_HANDLE = "ScrollpHATHD"
PACKAGE_MODULE = "scrollphathd"

# Prompte /usr/local/lib to the front of sys.path
#sys.path.insert(0,site.getsitepackages()[0])

import sphinx_rtd_theme

MOCK_MODULES = ['smbus','numpy']
for module_name in MOCK_MODULES:
    sys.modules[module_name] = mock.MagicMock()

sys.path.insert(0, '../library/')


import scrollphathd

from sphinx.ext import autodoc

class OutlineClassDocumenter(autodoc.ClassDocumenter):
    objtype = 'class'

    def add_content(self, more_content, no_docstring=False):
        return

class OutlineMethodDocumenter(autodoc.MethodDocumenter):
    objtype = 'method'

    def add_content(self, more_content, no_docstring=False):
        return

    def add_directive_header(self, sig):
        if self.objpath[0] == u"Matrix":
            self.objpath[0] = u'scrollphathd'
        autodoc.MethodDocumenter.add_directive_header(self, sig)

class OutlineFunctionDocumenter(autodoc.FunctionDocumenter):
    objtype = 'function'

    def add_content(self, more_content, no_docstring=False):
        return

    def add_directive_header(self, sig):
        if self.objpath[0] == u"Matrix":
            self.objpath[0] = u'scrollphathd'
        autodoc.FunctionDocumenter.add_directive_header(self, sig)

class MethodDocumenter(autodoc.MethodDocumenter):
    objtype = 'method'

    def add_directive_header(self, sig):
        if self.objpath[0] == u"Matrix":
            self.objpath[0] = u'scrollphathd'
        autodoc.MethodDocumenter.add_directive_header(self, sig)

class ClassOutlineDocumenter(autodoc.ClassDocumenter):
    objtype = 'classoutline'

    def add_directive_header(self, sig):
        pass # Squash directive header for At A Glance view

    def __init__(self, directive, name, indent=u''):
        # Monkey patch the Method and Function documenters
        sphinx_app.add_autodocumenter(OutlineMethodDocumenter)
        #sphinx_app.add_autodocumenter(OutlineClassDocumenter)
        sphinx_app.add_autodocumenter(OutlineFunctionDocumenter)
        autodoc.ClassDocumenter.__init__(self, directive, name, indent)

    def __del__(self):
        # Return the Method and Function documenters to normal
        #sphinx_app.add_autodocumenter(autodoc.ClassDocumenter)
        sphinx_app.add_autodocumenter(MethodDocumenter)
        sphinx_app.add_autodocumenter(autodoc.FunctionDocumenter)


def setup(app):
    global sphinx_app
    sphinx_app = app
    app.add_autodocumenter(ClassOutlineDocumenter)
    app.add_autodocumenter(MethodDocumenter)
    ClassOutlineDocumenter.objtype = 'class'

# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.viewcode',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The encoding of source files.
#
# source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = PACKAGE_NAME
copyright = u'2017, Pimoroni Ltd'
author = u'Phil Howard'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = u'{}'.format(scrollphathd.__version__)
# The full version, including alpha/beta/rc tags.
release = u'{}'.format(scrollphathd.__version__)

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#
# today = ''
#
# Else, today_fmt is used as the format for a strftime call.
#
# today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The reST default role (used for this markup: `text`) to use for all
# documents.
#
# default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#
# add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#
# add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#
# show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
# modindex_common_prefix = []

# If true, keep warnings as "system message" paragraphs in the built documents.
# keep_warnings = False

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'
#html_theme = 'alabaster'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
html_theme_options = {
    'collapse_navigation': False,
    'display_version': True
}

# Add any paths that contain custom themes here, relative to this directory.
html_theme_path = [
    '_themes',
    sphinx_rtd_theme.get_html_theme_path()
]

# The name for this set of Sphinx documents.
# "<project> v<release> documentation" by default.
#
# html_title = PACKAGE_NAME + u' v0.1.2'

# A shorter title for the navigation bar.  Default is the same as html_title.
#
# html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#
html_logo = 'shop-logo.png'

# The name of an image file (relative to this directory) to use as a favicon of
# the docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#
html_favicon = 'favicon.png'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# Add any extra paths that contain custom files (such as robots.txt or
# .htaccess) here, relative to this directory. These files are copied
# directly to the root of the documentation.
#
# html_extra_path = []

# If not None, a 'Last updated on:' timestamp is inserted at every page
# bottom, using the given strftime format.
# The empty string is equivalent to '%b %d, %Y'.
#
# html_last_updated_fmt = None

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#
# html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
#
# html_sidebars = {}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#
# html_additional_pages = {}

# If false, no module index is generated.
#
# html_domain_indices = True

# If false, no index is generated.
#
html_use_index = False

# If true, the index is split into individual pages for each letter.
#
# html_split_index = False

# If true, links to the reST sources are added to the pages.
#
html_show_sourcelink = False

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#
html_show_sphinx = False

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#
# html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#
# html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
# html_file_suffix = None

# Language to be used for generating the HTML full-text search index.
# Sphinx supports the following languages:
#   'da', 'de', 'en', 'es', 'fi', 'fr', 'hu', 'it', 'ja'
#   'nl', 'no', 'pt', 'ro', 'ru', 'sv', 'tr', 'zh'
#
# html_search_language = 'en'

# A dictionary with options for the search language support, empty by default.
# 'ja' uses this config value.
# 'zh' user can custom change `jieba` dictionary path.
#
# html_search_options = {'type': 'default'}

# The name of a javascript file (relative to the configuration directory) that
# implements a search results scorer. If empty, the default will be used.
#
# html_search_scorer = 'scorer.js'

# Output file base name for HTML help builder.
htmlhelp_basename = PACKAGE_HANDLE + 'doc'

# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
     # The paper size ('letterpaper' or 'a4paper').
     #
     # 'papersize': 'letterpaper',

     # The font size ('10pt', '11pt' or '12pt').
     #
     # 'pointsize': '10pt',

     # Additional stuff for the LaTeX preamble.
     #
     # 'preamble': '',

     # Latex figure (float) alignment
     #
     # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, PACKAGE_HANDLE + '.tex', PACKAGE_NAME + u' Documentation',
     u'Phil Howard', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#
# latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#
# latex_use_parts = False

# If true, show page references after internal links.
#
# latex_show_pagerefs = False

# If true, show URL addresses after external links.
#
# latex_show_urls = False

# Documents to append as an appendix to all manuals.
#
# latex_appendices = []

# It false, will not define \strong, \code, 	itleref, \crossref ... but only
# \sphinxstrong, ..., \sphinxtitleref, ... To help avoid clash with user added
# packages.
#
# latex_keep_old_macro_names = True

# If false, no module index is generated.
#
# latex_domain_indices = True


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc,  PACKAGE_MODULE, PACKAGE_NAME + u' Documentation',
     [author], 1)
]

# If true, show URL addresses after external links.
#
# man_show_urls = False


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, PACKAGE_HANDLE, PACKAGE_NAME + u' Documentation',
     author, PACKAGE_HANDLE, 'One line description of project.',
     'Miscellaneous'),
]

# Documents to append as an appendix to all manuals.
#
# texinfo_appendices = []

# If false, no module index is generated.
#
# texinfo_domain_indices = True

# How to display URL addresses: 'footnote', 'no', or 'inline'.
#
# texinfo_show_urls = 'footnote'

# If true, do not generate a @detailmenu in the "Top" node's menu.
#
# texinfo_no_detailmenu = False
